Kingdom_Class:
    type: data

    class:
        methods:
            constructor:
                definitions: name[ElementTag(String)]|constantCodeName[?ElementTag(Boolean)]
                script:
                - define **constantCodeName <[constantCodeName].if_null[false]>
                - define **codeName null

                - run Method def.call:<queue>.**self.SetName def.name:<[name]>

                - define *description <element[Default Description]>
                - define **warStatus false

                - define **members <list[]>
                - define **upkeep <map[]>
                - define **warps <map[]>

                - define **balance 0
                - define **prestige 0

            SetName:
                definitions: name[ElementTag(String)]
                script:
                # This prevents the user from fucking up the next if check.
                - if <[name]> == null:
                    - stop

                - define **displayName <[name]>

                # If **constantCodeName is set to true then this method will only set the code
                # name as something based on the actual name during the object's initial setup.
                # Afterwards, changing the code name needs to be done as a separate method call to
                # the dedicated SetCodeName() method.
                - if <[**constantCodeName]> && <[**codeName]> == null:
                    - run Method def.call:<queue>.**self.SetCodeName def.name:<[name]>

            SetCodeName:
                definitions: name[ElementTag(String)]
                script:
                - if <[name]> == null:
                    - stop

                - define **codeName <[name].replace_text[ ].with[-].escaped>

            SetShortName:
                definitions: shortName[ElementTag(String)]
                script:
                - define **shortName <[shortName]>

            GetName:
                script:
                - determine <[**displayName]>

            GetCodeName:
                script:
                - determine <[**codeName]>

            GetShortName:
                script:
                - determine <[**shortName]>

            SetWarStatus:
                definitions: status[ElementTag(Boolean)]
                script:
                - if !<[status].is_boolean>:
                    - debug ERROR <element[Cannot set kingdom war status to a non-boolean value!]>
                    - determine cancelled

                - define **warStatus <[status]>

            GetWarStatus:
                script:
                - determine <[**warStatus]>

            SetBalance:
                definitions: newBalance[ElementTag(Float)]
                script:
                - if !<[newBalance].is_decimal>:
                    - debug ERROR <element[Cannot set kingdom balance to a non-decimal value!]>
                    - determine cancelled

                - define **balance <[newBalance]>

            GetBalance:
                script:
                - determine <[**balance]>

            SetPrestige:
                definitions: newPrestige[ElementTag(Float)]
                script:
                - if !<[newPrestige].is_decimal>:
                    - debug ERROR <element[Cannot set kingdom prestige to a non-decimal value!]>
                    - determine cancelled

                - define **prestige <[newPrestige]>

            GetPrestige:
                script:
                - determine <[**prestige]>

            GetMembers:
                script:
                - determine <[**members]>

            AddMember:
                definitions: player[PlayerTag]
                script:
                - if !<[player].as[player].is_in[<server.players>]>:
                    - debug ERROR <element[No player exists with matcher: <[player]>]>
                    - determine cancelled

                - define **members:->:<[player]>
                - flag <[player]> kingdom:<[**codeName]>

            RemoveMember:
                definitions: player[PlayerTag]
                script:
                - if !<[player].as[player].is_in[<server.players>]>:
                    - debug ERROR <element[No player exists with matcher: <[player]>]>
                    - determine cancelled

                - if !<[**members].contains[<[player]>]>:
                    - debug LOG <element[Player: <[player]> is already not part of the kingdom]>
                    - determine cancelled

                - define **members:->:<[player]>
                - flag <[player]> kingdom:<[**codeName]>


TestKingdomClass:
    type: task
    script:
    - run Object def.queue:<queue> def.class:Kingdom_Class def.object:talpenhern def.name:Talpenhern

    - if <player.exists>:
        - run Method def:<queue>.talpenhern.AddMember def.player:<player>
        - run Method def:<queue>.talpenhern.GetMembers save:allMembers

        - narrate <entry[allMembers].created_queue.determination.get[1].parse_tag[<[parse_value].name>]>

    - run Method def:<queue>.talpenhern.__class save:class
    - narrate <entry[class].created_queue.determination.get[1]>

    - run Method def:<queue>.talpenhern.__str save:str
    - narrate <entry[str].created_queue.determination.get[1]>

    - narrate IS_INSTANCE:<proc[IsInstance].context[<queue>|talpenhern|Kingdom_Class]>
    - narrate IS_CLASS:<proc[IsClass].context[<queue.flag[dclasses.talpenhern]>]>
