SamplePlayer_Class:
    type: data
    class:
        # The 'attributes' key refers not to the instance-level attributes that are defined in the
        # constructor of the class, but instead to any class attributes that the user wishes to
        # define.
        #
        # This section is structured as a YAML map, not an executable script since I don't want the
        # user to start treating this as another constructor section. It is not.
        attributes:
            baseSpawnLoc: <location[0,65,0,0,90,<server.worlds.get[1].name>]>

        methods:
            # This will obviously be a reserved word. You can't name a method 'constructor'
            constructor:
                definitions: name[ElementTag(String)]
                script:
                # The way to define a regular attribute in a class is to start the name of its
                # definition with an *. This way dClasses knows that this is not just a regular
                # definition in a method.
                - define *name:<[name]>
                - define *health:20

                # Adding a ** before the name of a definition indicates that it is private and can
                # only be accessed from within the class.
                - define **uuid:123

                # Adding a *** before the name of a definition indicates that it is protected and
                # can only be accessed from within the class or a subclass.
                - define ***protectedValue 456

                # Adding a ^ before the name of a definition indicates that is referencing a global
                # attribute by the same name (excluding the ^).
                - define ^baseSpawnLoc <[^baseSpawnLoc].add[0,10,0]>

            RunAMethod:
                script:
                - run Method def.queue:<queue> def.method:~*PrivateMethod def.object:**self save:uuid
                - narrate <entry[uuid].created_queue.determination.get[1]>

            ~*PrivateMethod:
                script:
                - narrate <element[I do the same thing as GetUUID]>
                - run Method def.queue:<queue> def.method:GetUUID def.object:**self save:uuid
                - determine <entry[uuid].created_queue.determination.get[1]>

            GetUUID:
                script:
                - determine <[**uuid]>

            SetUUID:
                script:
                - narrate <element[You cannot set a player uuid!]>

            GetHealth:
                script:
                - determine <[*health]>

            SetHealth:
                definitions: newHealth
                script:
                - define *health <[newHealth]>


SampleKingdom_Class:
    type: data
    class:
        methods:
            constructor:
                definitions: name[ElementTag(String)]|players[ListTag(SamplePlayer_Class)]|prestige[ElementTag(Float)]
                script:
                - define players <list[]> if:<[players].exists.not>
                - define prestige 0 if:<[prestige].exists.not>

                - define *name <[name]>
                - define *players <[players]>
                - define *balance 0
                - define *upkeep 0
                - define *prestige <[prestige]>

            AddPlayer:
                definitions: player[SamplePlayer_Class]
                script:
                - define *players:->:<[player]>


TestClassScript:
    type: task
    script:
    - run Object def.queue:<queue> def.class:SamplePlayer_Class def.object:newPlayer def.name:ZyadTheBoss
    - narrate <proc[GetAttribute].context[<queue>|newPlayer|name]>

    - run Method def.queue:<queue> def.object:newPlayer def.method:RunAMethod
    # - run SetAttribute def.queue:<queue> def.object:newPlayer def.attribute:name def.value:ZyadTheManager
    # - narrate <proc[GetAttribute].context[<queue>|newPlayer|name]>

    # - narrate <proc[GetAttribute].context[<queue>|newPlayer|health]>

    # - run Method def.queue:<queue> def.object:newPlayer def.method:SetHealth def.newHealth:18
    # - narrate <proc[GetAttribute].context[<queue>|newPlayer|health]>

    # - run Object def.queue:<queue> def.class:SampleKingdom_Class def.object:centran def.name:<element[Centra Australis]>
    # - run Method def.queue:<queue> def.object:centran def.method:AddPlayer def.player:<proc[ObjectRef].context[<queue>|newPlayer]>

    # - narrate ----

    # - run flagvisualizer def.flag:<proc[GetAttribute].context[<queue>|centran|players]>

    # - define newPlayer <proc[Object].context[SamplePlayer_Class|ZyadTheBoss]>
    # - narrate <proc[GetAttribute].context[<[newPlayer]>|name]>

    # - definemap testMap:
    #     _newPlayer: <proc[Object].context[SamplePlayer_Class|ZyadTheBoss]>
    #     something: <element[something]>
    #     else: <element[else]>

    # - define subList <list[1|<proc[Object].context[SamplePlayer_Class|ZyadTheBoss]>]>
    # - define newPlayerTwo <list[1|2].include_single[<[subList]>]>
    # - narrate <proc[GetAttribute].context[<[newPlayer]>|name]>

    # - narrate <[newPlayerTwo].get[3]>

    # - run Method def.queue:<queue> def.object:<[newPlayer]> def.method:SetHealth def.newHealth:18
    # - run Method def.queue:<queue> def.object:<[newPlayerTwo].get[3].get[2]> def.method:GetUUID save:uuid
    # - narrate <entry[uuid].created_queue.determination.get[1]>
